# Uniformly Weighted LASSO type quantile regression at a given quantile index vector with BIC tuning parameter selector
#'@export
vecrqalasso=function(y,x,taus, method="BC", weights="unif", grid=seq(0,nrow(x)/20, length.out=nrow(X)/4), tol=1e-6){
  ok<-complete.cases(x,y);
  x<-x[ok,]                            # get rid of na's
  y<-y[ok]                             # since regsubsets can't handle na's
  p=ncol(x);                           # Dimensionality
  n=nrow(x);                           # sample size
  x<-as.matrix(x);                     # in case x is not a matrix
  
  #### standardize y

  meany=mean(y);
  Y=y-meany;    
  normy=sqrt(mean(Y^2));
  Y=Y/normy;
  
  #### standardize x
  meanx=apply(x,2,mean);
  X=x-outer(rep(1,dim(x)[1]),meanx);
  normx=sqrt(apply(X^2,2,mean));
  X=sweep(X,2,1/normx,"*");
  
  Z=cbind(rep(1,n),X);
  ####  tuning parameter for the initial estimator following  Belloni and Chernozhukov (2011)
  if (method=="BC"){
    Rlambda=rep(0,n);
    for(i in 1:n){
      U=runif(n,0,1);                                    ###### generate n uniform r.v.s
      tmpLamb=rep(0,length(taus));                          ###### find maximum with respect to j's
      for(k in 1:length(taus)){
        u=taus[k];
        tmpLamb[k]=max(abs((u-(U<=u))%*%X/sqrt(u*(1-u))));
      }
      Rlambda[i]=max(tmpLamb);           ######## maximize with respect to U's
    }
    lambda=quantile(Rlambda,0.2)*0.4;  
  }
  ####  tuning parameter for the initial estimator using 5-fold cross-validation 
  if (method=="CV5"){
    lambda=lambdaCV(Y,Z,taus, tol=tol);
  }
   
  ####  obtain initial estimator 
  lambda1=c(0,rep(lambda,p))
  fitb=coef(rq(Y~0+Z,method="lasso",tau=taus,lambda=lambda1));
  Estimate=t(fitb*(abs(fitb)>=tol));
  
  ####  compute the adaptive weights
  Pilot=1/abs(Estimate);   
  if (weights=="unif"){
    w=t(replicate(length(taus), apply(Pilot,2,min)));
  } 
  if (weights=="pointwise"){
    w=Pilot;
  }  
  if (weights=="absmean"){  
    pvec=apply(abs(Estimate^5),2,sum);
    numvec=apply(abs(Estimate^4),2,sum);
    w=pvec/numvec;
    w[which(w=="NaN")]=0;
    w=t(replicate(length(taus), 1/w));
  }
  w[which(w>1/tol)]=1/tol;
  
  #### tuning parameter for adaptive estimator
  biclambda=lambdaGIC(Y,Z,tau=taus, w, grid, tol);
  
  ####  obtain adaptive estimator 
  betafit=matrix(0, length(taus),dim(Z)[2]); 
  for(s in 1:length(taus)){
    betafit[s,]=coef(rq(Y~0+Z,method="lasso",tau=taus[s],lambda=biclambda*w[s,]));
    betafit[s,]=betafit[s,]*(abs(betafit[s,])>=tol);
  }
  normx=c(1,normx);
  fitted=scale(betafit,FALSE,normx)*normy;
  obj=list(
    taus=taus,
    estbeta=fitted,
    lambda.cv=lambda,
    lambda.gic=biclambda,
    method=method,
    weights=weights
    );
  return(obj);
}
